@@ -17,10 +17,13 @@
 
 package org.apache.commons.csv;
 
+import static org.apache.commons.csv.Constants.BACKSLASH;
 import static org.apache.commons.csv.Constants.CR;
 import static org.junit.jupiter.api.Assertions.assertArrayEquals;
 import static org.junit.jupiter.api.Assertions.assertEquals;
 import static org.junit.jupiter.api.Assertions.assertFalse;
+import static org.junit.jupiter.api.Assertions.assertNotEquals;
+import static org.junit.jupiter.api.Assertions.assertNull;
 import static org.junit.jupiter.api.Assertions.assertThrows;
 import static org.mockito.Mockito.mock;
 import static org.mockito.Mockito.never;
@@ -29,8 +32,10 @@ import static org.mockito.Mockito.verify;
 
 import java.io.CharArrayWriter;
 import java.io.File;
+import java.io.FileReader;
 import java.io.IOException;
 import java.io.PrintStream;
+import java.io.Reader;
 import java.io.StringReader;
 import java.io.StringWriter;
 import java.io.Writer;
@@ -60,15 +65,14 @@ import org.junit.jupiter.api.Disabled;
 import org.junit.jupiter.api.Test;
 
 /**
- *
+ * Tests {@link CSVPrinter}.
  */
 public class CSVPrinterTest {
 
-    private static final char EURO_CH = '\u20AC';
     private static final char DQUOTE_CHAR = '"';
-    private static final char BACKSLASH_CH = '\\';
-    private static final char QUOTE_CH = '\'';
+    private static final char EURO_CH = '\u20AC';
     private static final int ITERATIONS_FOR_RANDOM_TEST = 50000;
+    private static final char QUOTE_CH = '\'';
 
     private static String printable(final String s) {
         final StringBuilder sb = new StringBuilder();
@@ -83,9 +87,10 @@ public class CSVPrinterTest {
         return sb.toString();
     }
 
-    private final String recordSeparator = CSVFormat.DEFAULT.getRecordSeparator();
     private String longText2;
 
+    private final String recordSeparator = CSVFormat.DEFAULT.getRecordSeparator();
+
     private void doOneRandom(final CSVFormat format) throws Exception {
         final Random r = new Random();
 
@@ -138,11 +143,6 @@ public class CSVPrinterTest {
         return fixed;
     }
 
-    private Connection geH2Connection() throws SQLException, ClassNotFoundException {
-        Class.forName("org.h2.Driver");
-        return DriverManager.getConnection("jdbc:h2:mem:my_test;", "sa", "");
-    }
-
     private String[][] generateLines(final int nLines, final int nCol) {
         final String[][] lines = new String[nLines][];
         for (int i = 0; i < nLines; i++) {
@@ -155,13 +155,21 @@ public class CSVPrinterTest {
         return lines;
     }
 
+    private Connection getH2Connection() throws SQLException, ClassNotFoundException {
+        Class.forName("org.h2.Driver");
+        return DriverManager.getConnection("jdbc:h2:mem:my_test;", "sa", "");
+    }
+
     private CSVPrinter printWithHeaderComments(final StringWriter sw, final Date now, final CSVFormat baseFormat)
             throws IOException {
-        CSVFormat format = baseFormat;
         // Use withHeaderComments first to test CSV-145
-        format = format.withHeaderComments("Generated by Apache Commons CSV 1.1", now);
-        format = format.withCommentMarker('#');
-        format = format.withHeader("Col1", "Col2");
+        // @formatter:off
+        final CSVFormat format = baseFormat.builder()
+                .setHeaderComments("Generated by Apache Commons CSV 1.1", now)
+                .setCommentMarker('#')
+                .setHeader("Col1", "Col2")
+                .build();
+        // @formatter:on
         final CSVPrinter csvPrinter = format.print(sw);
         csvPrinter.printRecord("A", "B");
         csvPrinter.printRecord("C", "D");
@@ -177,7 +185,7 @@ public class CSVPrinterTest {
         final char[] buf = new char[sz];
         for (int i = 0; i < sz; i++) {
             // stick in special chars with greater frequency
-            char ch;
+            final char ch;
             final int what = r.nextInt(20);
             switch (what) {
             case 0:
@@ -205,7 +213,7 @@ public class CSVPrinterTest {
                 ch = '\'';
                 break;
             case 8:
-                ch = BACKSLASH_CH;
+                ch = BACKSLASH;
                 break;
             default:
                 ch = (char) r.nextInt(300);
@@ -279,14 +287,25 @@ public class CSVPrinterTest {
     @Test
     public void testCloseWithFlushOn() throws IOException {
         try (final Writer writer = mock(Writer.class)) {
-            final CSVFormat csvFormat = CSVFormat.DEFAULT;
             @SuppressWarnings("resource")
-            final CSVPrinter csvPrinter = new CSVPrinter(writer, csvFormat);
+            final CSVPrinter csvPrinter = new CSVPrinter(writer, CSVFormat.DEFAULT);
             csvPrinter.close(true);
             verify(writer, times(1)).flush();
         }
     }
 
+    @Test
+    public void testCRComment() throws IOException {
+        final StringWriter sw = new StringWriter();
+        final Object value = "abc";
+        try (final CSVPrinter printer = new CSVPrinter(sw, CSVFormat.DEFAULT.withCommentMarker('#'))) {
+            printer.print(value);
+            printer.printComment("This is a comment\r\non multiple lines\rthis is next comment\r");
+            assertEquals("abc" + recordSeparator + "# This is a comment" + recordSeparator + "# on multiple lines"
+                        + recordSeparator + "# this is next comment" + recordSeparator + "# " + recordSeparator, sw.toString());
+        }
+    }
+
     @Test
     public void testCSV135() throws IOException {
         final List<String> list = new LinkedList<>();
@@ -310,6 +329,16 @@ public class CSVPrinterTest {
         tryFormat(list, '"',  '"',  "\"\"\"\"\"\",\\\\,\"\\\"\"\\\"");
     }
 
+    @Test
+    public void testCSV259() throws IOException {
+        final StringWriter sw = new StringWriter();
+        try (final Reader reader = new FileReader("src/test/resources/org/apache/commons/csv/CSV-259/sample.txt");
+                final CSVPrinter printer = new CSVPrinter(sw, CSVFormat.DEFAULT.withEscape('!').withQuote(null))) {
+            printer.print(reader);
+            assertEquals("x!,y!,z", sw.toString());
+        }
+    }
+
     @Test
     public void testDelimeterQuoted() throws IOException {
         final StringWriter sw = new StringWriter();
@@ -331,6 +360,28 @@ public class CSVPrinterTest {
         }
     }
 
+    @Test
+    public void testDelimeterStringQuoted() throws IOException {
+        final StringWriter sw = new StringWriter();
+        try (final CSVPrinter printer = new CSVPrinter(sw, CSVFormat.DEFAULT.builder().setDelimiter("[|]").setQuote('\'').build())) {
+            printer.print("a[|]b[|]c");
+            printer.print("xyz");
+            assertEquals("'a[|]b[|]c'[|]xyz", sw.toString());
+        }
+    }
+
+    @Test
+    public void testDelimeterStringQuoteNone() throws IOException {
+        final StringWriter sw = new StringWriter();
+        final CSVFormat format = CSVFormat.DEFAULT.builder().setDelimiter("[|]").setEscape('!').setQuoteMode(QuoteMode.NONE).build();
+        try (final CSVPrinter printer = new CSVPrinter(sw, format)) {
+            printer.print("a[|]b[|]c");
+            printer.print("xyz");
+            printer.print("a[xy]bc[]");
+            assertEquals("a![!|!]b![!|!]c[|]xyz[|]a[xy]bc[]", sw.toString());
+        }
+    }
+
     @Test
     public void testDelimiterEscaped() throws IOException {
         final StringWriter sw = new StringWriter();
@@ -351,6 +402,16 @@ public class CSVPrinterTest {
         }
     }
 
+    @Test
+    public void testDelimiterStringEscaped() throws IOException {
+        final StringWriter sw = new StringWriter();
+        try (final CSVPrinter printer = new CSVPrinter(sw, CSVFormat.DEFAULT.builder().setDelimiter("|||").setEscape('!').setQuote(null).build())) {
+            printer.print("a|||b|||c");
+            printer.print("xyz");
+            assertEquals("a!|!|!|b!|!|!|c|||xyz", sw.toString());
+        }
+    }
+
     @Test
     public void testDisabledComment() throws IOException {
         final StringWriter sw = new StringWriter();
@@ -522,7 +583,7 @@ public class CSVPrinterTest {
         final StringWriter sw = new StringWriter();
         try (final CSVPrinter printer = new CSVPrinter(sw, CSVFormat.EXCEL)) {
             printer.printRecords(
-                    Arrays.asList(new List[] { Arrays.asList("r1c1", "r1c2"), Arrays.asList("r2c1", "r2c2") }));
+                    Arrays.asList(Arrays.asList("r1c1", "r1c2"), Arrays.asList("r2c1", "r2c2")));
             assertEquals("r1c1,r1c2" + recordSeparator + "r2c1,r2c2" + recordSeparator, sw.toString());
         }
     }
@@ -596,22 +657,22 @@ public class CSVPrinterTest {
     @Test
     public void testJdbcPrinter() throws IOException, ClassNotFoundException, SQLException {
         final StringWriter sw = new StringWriter();
-        try (final Connection connection = geH2Connection()) {
+        try (final Connection connection = getH2Connection()) {
             setUpTable(connection);
             try (final Statement stmt = connection.createStatement();
-                    final CSVPrinter printer = new CSVPrinter(sw, CSVFormat.DEFAULT)) {
-                printer.printRecords(stmt.executeQuery("select ID, NAME, TEXT from TEST"));
+                    final CSVPrinter printer = new CSVPrinter(sw, CSVFormat.DEFAULT);
+                    final ResultSet resultSet = stmt.executeQuery("select ID, NAME, TEXT from TEST");) {
+                printer.printRecords(resultSet);
             }
         }
-        assertEquals("1,r1,\"long text 1\"" + recordSeparator + "2,r2,\"" + longText2 + "\"" + recordSeparator,
-                sw.toString());
+        assertEquals("1,r1,\"long text 1\"" + recordSeparator + "2,r2,\"" + longText2 + "\"" + recordSeparator, sw.toString());
     }
 
     @Test
     public void testJdbcPrinterWithResultSet() throws IOException, ClassNotFoundException, SQLException {
         final StringWriter sw = new StringWriter();
         Class.forName("org.h2.Driver");
-        try (final Connection connection = geH2Connection();) {
+        try (final Connection connection = getH2Connection()) {
             setUpTable(connection);
             try (final Statement stmt = connection.createStatement();
                     final ResultSet resultSet = stmt.executeQuery("select ID, NAME, TEXT from TEST");
@@ -623,11 +684,32 @@ public class CSVPrinterTest {
                 + "\"" + recordSeparator, sw.toString());
     }
 
+    @Test
+    public void testJdbcPrinterWithResultSetHeader() throws IOException, ClassNotFoundException, SQLException {
+        final StringWriter sw = new StringWriter();
+        try (final Connection connection = getH2Connection()) {
+            setUpTable(connection);
+            try (final Statement stmt = connection.createStatement();
+                final CSVPrinter printer = new CSVPrinter(sw, CSVFormat.DEFAULT);) {
+                try (final ResultSet resultSet = stmt.executeQuery("select ID, NAME from TEST")) {
+                    printer.printRecords(resultSet, true);
+                    assertEquals("ID,NAME" + recordSeparator + "1,r1" + recordSeparator + "2,r2" + recordSeparator,
+                        sw.toString());
+                }
+                try (final ResultSet resultSet = stmt.executeQuery("select ID, NAME from TEST")) {
+                    printer.printRecords(resultSet, false);
+                    assertNotEquals("ID,NAME" + recordSeparator + "1,r1" + recordSeparator + "2,r2" + recordSeparator,
+                        sw.toString());
+                }
+            }
+        }
+    }
+
     @Test
     public void testJdbcPrinterWithResultSetMetaData() throws IOException, ClassNotFoundException, SQLException {
         final StringWriter sw = new StringWriter();
         Class.forName("org.h2.Driver");
-        try (final Connection connection = geH2Connection()) {
+        try (final Connection connection = getH2Connection()) {
             setUpTable(connection);
             try (final Statement stmt = connection.createStatement();
                     final ResultSet resultSet = stmt.executeQuery("select ID, NAME, TEXT from TEST");
@@ -642,7 +724,7 @@ public class CSVPrinterTest {
     @Test
     @Disabled
     public void testJira135_part1() throws IOException {
-        final CSVFormat format = CSVFormat.DEFAULT.withRecordSeparator('\n').withQuote(DQUOTE_CHAR).withEscape(BACKSLASH_CH);
+        final CSVFormat format = CSVFormat.DEFAULT.withRecordSeparator('\n').withQuote(DQUOTE_CHAR).withEscape(BACKSLASH);
         final StringWriter sw = new StringWriter();
         final List<String> list = new LinkedList<>();
         try (final CSVPrinter printer = new CSVPrinter(sw, format)) {
@@ -658,7 +740,7 @@ public class CSVPrinterTest {
     @Test
     @Disabled
     public void testJira135_part2() throws IOException {
-        final CSVFormat format = CSVFormat.DEFAULT.withRecordSeparator('\n').withQuote(DQUOTE_CHAR).withEscape(BACKSLASH_CH);
+        final CSVFormat format = CSVFormat.DEFAULT.withRecordSeparator('\n').withQuote(DQUOTE_CHAR).withEscape(BACKSLASH);
         final StringWriter sw = new StringWriter();
         final List<String> list = new LinkedList<>();
         try (final CSVPrinter printer = new CSVPrinter(sw, format)) {
@@ -674,7 +756,7 @@ public class CSVPrinterTest {
     @Test
     @Disabled
     public void testJira135_part3() throws IOException {
-        final CSVFormat format = CSVFormat.DEFAULT.withRecordSeparator('\n').withQuote(DQUOTE_CHAR).withEscape(BACKSLASH_CH);
+        final CSVFormat format = CSVFormat.DEFAULT.withRecordSeparator('\n').withQuote(DQUOTE_CHAR).withEscape(BACKSLASH);
         final StringWriter sw = new StringWriter();
         final List<String> list = new LinkedList<>();
         try (final CSVPrinter printer = new CSVPrinter(sw, format)) {
@@ -690,7 +772,7 @@ public class CSVPrinterTest {
     @Test
     @Disabled
     public void testJira135All() throws IOException {
-        final CSVFormat format = CSVFormat.DEFAULT.withRecordSeparator('\n').withQuote(DQUOTE_CHAR).withEscape(BACKSLASH_CH);
+        final CSVFormat format = CSVFormat.DEFAULT.withRecordSeparator('\n').withQuote(DQUOTE_CHAR).withEscape(BACKSLASH);
         final StringWriter sw = new StringWriter();
         final List<String> list = new LinkedList<>();
         try (final CSVPrinter printer = new CSVPrinter(sw, format)) {
@@ -791,7 +873,7 @@ public class CSVPrinterTest {
         String expected = "\"NULL\"\tNULL\n";
         assertEquals(expected, writer.toString());
         String[] record0 = toFirstRecordValues(expected, format);
-        assertArrayEquals(new Object[2], record0);
+        assertArrayEquals(s, record0);
 
         s = new String[] { "\\N", null };
         format = CSVFormat.MYSQL.withNullString("\\N");
@@ -889,12 +971,22 @@ public class CSVPrinterTest {
 
     @Test
     public void testNewCsvPrinterAppendableNullFormat() {
-        assertThrows(IllegalArgumentException.class, () -> new CSVPrinter(new StringWriter(), null));
+        assertThrows(NullPointerException.class, () -> new CSVPrinter(new StringWriter(), null));
     }
 
     @Test
     public void testNewCsvPrinterNullAppendableFormat() {
-        assertThrows(IllegalArgumentException.class, () -> new CSVPrinter(null, CSVFormat.DEFAULT));
+        assertThrows(NullPointerException.class, () -> new CSVPrinter(null, CSVFormat.DEFAULT));
+    }
+
+    @Test
+    public void testNotFlushable() throws IOException {
+        final Appendable out = new StringBuilder();
+        try (final CSVPrinter printer = new CSVPrinter(out, CSVFormat.DEFAULT)) {
+            printer.printRecord("a", "b", "c");
+            assertEquals("a,b,c" + recordSeparator, out.toString());
+            printer.flush();
+        }
     }
 
     @Test
@@ -910,7 +1002,7 @@ public class CSVPrinterTest {
             final Iterator<CSVRecord> iterator = iterable.iterator();
             final CSVRecord record = iterator.next();
             assertEquals("a", record.get(0));
-            assertEquals(null, record.get(1));
+            assertNull(record.get(1));
             assertEquals("b", record.get(2));
             assertFalse(iterator.hasNext());
         }
@@ -1170,6 +1262,68 @@ public class CSVPrinterTest {
         }
     }
 
+    @Test
+    public void testPrintCSVParser() throws IOException {
+        final String code = "a1,b1\n" // 1)
+            + "a2,b2\n" // 2)
+            + "a3,b3\n" // 3)
+            + "a4,b4\n"// 4)
+        ;
+        final String[][] res = {{"a1", "b1"}, {"a2", "b2"}, {"a3", "b3"}, {"a4", "b4"}};
+        final CSVFormat format = CSVFormat.DEFAULT;
+        final StringWriter sw = new StringWriter();
+        try (final CSVPrinter printer = format.print(sw); final CSVParser parser = CSVParser.parse(code, format)) {
+            printer.printRecords(parser);
+        }
+        try (final CSVParser parser = CSVParser.parse(sw.toString(), format)) {
+            final List<CSVRecord> records = parser.getRecords();
+            assertFalse(records.isEmpty());
+            Utils.compare("Fail", res, records);
+        }
+    }
+
+    @Test
+    public void testPrintCSVRecord() throws IOException {
+        final String code = "a1,b1\n" // 1)
+            + "a2,b2\n" // 2)
+            + "a3,b3\n" // 3)
+            + "a4,b4\n"// 4)
+        ;
+        final String[][] res = {{"a1", "b1"}, {"a2", "b2"}, {"a3", "b3"}, {"a4", "b4"}};
+        final CSVFormat format = CSVFormat.DEFAULT;
+        final StringWriter sw = new StringWriter();
+        try (final CSVPrinter printer = format.print(sw); final CSVParser parser = CSVParser.parse(code, format)) {
+            for (final CSVRecord record : parser) {
+                printer.printRecord(record);
+            }
+        }
+        try (final CSVParser parser = CSVParser.parse(sw.toString(), format)) {
+            final List<CSVRecord> records = parser.getRecords();
+            assertFalse(records.isEmpty());
+            Utils.compare("Fail", res, records);
+        }
+    }
+
+    @Test
+    public void testPrintCSVRecords() throws IOException {
+        final String code = "a1,b1\n" // 1)
+            + "a2,b2\n" // 2)
+            + "a3,b3\n" // 3)
+            + "a4,b4\n"// 4)
+        ;
+        final String[][] res = {{"a1", "b1"}, {"a2", "b2"}, {"a3", "b3"}, {"a4", "b4"}};
+        final CSVFormat format = CSVFormat.DEFAULT;
+        final StringWriter sw = new StringWriter();
+        try (final CSVPrinter printer = format.print(sw); final CSVParser parser = CSVParser.parse(code, format)) {
+            printer.printRecords(parser.getRecords());
+        }
+        try (final CSVParser parser = CSVParser.parse(sw.toString(), format)) {
+            final List<CSVRecord> records = parser.getRecords();
+            assertFalse(records.isEmpty());
+            Utils.compare("Fail", res, records);
+        }
+    }
+
     @Test
     public void testPrintCustomNullValues() throws IOException {
         final StringWriter sw = new StringWriter();
@@ -1302,6 +1456,21 @@ public class CSVPrinterTest {
         assertEquals(content, sw.toString());
     }
 
+    @Test
+    public void testPrintRecordsWithCSVRecord() throws IOException {
+        final String[] values = {"A", "B", "C"};
+        final String rowData = StringUtils.join(values, ',');
+        final CharArrayWriter charArrayWriter = new CharArrayWriter(0);
+        try (final CSVParser parser = CSVFormat.DEFAULT.parse(new StringReader(rowData));
+            final CSVPrinter csvPrinter = CSVFormat.INFORMIX_UNLOAD.print(charArrayWriter)) {
+            for (final CSVRecord record : parser) {
+                csvPrinter.printRecord(record);
+            }
+        }
+        assertEquals(6, charArrayWriter.size());
+        assertEquals("A|B|C" + CSVFormat.INFORMIX_UNLOAD.getRecordSeparator(), charArrayWriter.toString());
+    }
+
     @Test
     public void testPrintRecordsWithEmptyVector() throws IOException {
         final PrintStream out = System.out;
@@ -1435,6 +1604,7 @@ public class CSVPrinterTest {
         doRandom(CSVFormat.POSTGRESQL_TEXT, ITERATIONS_FOR_RANDOM_TEST);
     }
 
+
     @Test
     public void testRandomRfc4180() throws Exception {
         doRandom(CSVFormat.RFC4180, ITERATIONS_FOR_RANDOM_TEST);
@@ -1445,7 +1615,6 @@ public class CSVPrinterTest {
         doRandom(CSVFormat.TDF, ITERATIONS_FOR_RANDOM_TEST);
     }
 
-
     @Test
     public void testSingleLineComment() throws IOException {
         final StringWriter sw = new StringWriter();
@@ -1530,12 +1699,12 @@ public class CSVPrinterTest {
         return CSVParser.parse(expected, format).getRecords().get(0).values();
     }
 
-    private void tryFormat(final List<String> l, final Character quote, final Character escape, final String expected) throws IOException {
+    private void tryFormat(final List<String> list, final Character quote, final Character escape, final String expected) throws IOException {
         final CSVFormat format = CSVFormat.DEFAULT.withQuote(quote).withEscape(escape).withRecordSeparator(null);
         final Appendable out = new StringBuilder();
-        final CSVPrinter printer = new CSVPrinter(out, format);
-        printer.printRecord(l);
-        printer.close();
+        try (final CSVPrinter printer = new CSVPrinter(out, format)) {
+            printer.printRecord(list);
+        }
         assertEquals(expected, out.toString());
     }
 }
