@@ -29,6 +29,7 @@ import static org.apache.commons.csv.Token.Type.TOKEN;
 import static org.apache.commons.csv.TokenMatchers.hasContent;
 import static org.apache.commons.csv.TokenMatchers.matches;
 import static org.hamcrest.MatcherAssert.assertThat;
+import static org.junit.jupiter.api.Assertions.assertEquals;
 import static org.junit.jupiter.api.Assertions.assertFalse;
 import static org.junit.jupiter.api.Assertions.assertThrows;
 import static org.junit.jupiter.api.Assertions.assertTrue;
@@ -46,63 +47,65 @@ public class LexerTest {
 
     private CSVFormat formatWithEscaping;
 
-    @BeforeEach
-    public void setUp() {
-        formatWithEscaping = CSVFormat.DEFAULT.withEscape('\\');
-    }
-
     @SuppressWarnings("resource")
     private Lexer createLexer(final String input, final CSVFormat format) {
         return new Lexer(format, new ExtendedBufferedReader(new StringReader(input)));
     }
 
-    @Test
-    public void testSurroundingSpacesAreDeleted() throws IOException {
-        final String code = "noSpaces,  leadingSpaces,trailingSpaces  ,  surroundingSpaces  ,  ,,";
-        try (final Lexer parser = createLexer(code, CSVFormat.DEFAULT.withIgnoreSurroundingSpaces())) {
-            assertThat(parser.nextToken(new Token()), matches(TOKEN, "noSpaces"));
-            assertThat(parser.nextToken(new Token()), matches(TOKEN, "leadingSpaces"));
-            assertThat(parser.nextToken(new Token()), matches(TOKEN, "trailingSpaces"));
-            assertThat(parser.nextToken(new Token()), matches(TOKEN, "surroundingSpaces"));
-            assertThat(parser.nextToken(new Token()), matches(TOKEN, ""));
-            assertThat(parser.nextToken(new Token()), matches(TOKEN, ""));
-            assertThat(parser.nextToken(new Token()), matches(EOF, ""));
-        }
+    @BeforeEach
+    public void setUp() {
+        formatWithEscaping = CSVFormat.DEFAULT.withEscape('\\');
     }
 
+    // simple token with escaping enabled
     @Test
-    public void testSurroundingTabsAreDeleted() throws IOException {
-        final String code = "noTabs,\tleadingTab,trailingTab\t,\tsurroundingTabs\t,\t\t,,";
-        try (final Lexer parser = createLexer(code, CSVFormat.DEFAULT.withIgnoreSurroundingSpaces())) {
-            assertThat(parser.nextToken(new Token()), matches(TOKEN, "noTabs"));
-            assertThat(parser.nextToken(new Token()), matches(TOKEN, "leadingTab"));
-            assertThat(parser.nextToken(new Token()), matches(TOKEN, "trailingTab"));
-            assertThat(parser.nextToken(new Token()), matches(TOKEN, "surroundingTabs"));
-            assertThat(parser.nextToken(new Token()), matches(TOKEN, ""));
-            assertThat(parser.nextToken(new Token()), matches(TOKEN, ""));
-            assertThat(parser.nextToken(new Token()), matches(EOF, ""));
-        }
-    }
-
-    @Test
-    public void testIgnoreEmptyLines() throws IOException {
-        final String code = "first,line,\n" + "\n" + "\n" + "second,line\n" + "\n" + "\n" + "third line \n" + "\n" +
-                "\n" + "last, line \n" + "\n" + "\n" + "\n";
-        final CSVFormat format = CSVFormat.DEFAULT.withIgnoreEmptyLines();
+    public void testBackslashWithEscaping() throws IOException {
+        /*
+         * file: a,\,,b \,,
+         */
+        final String code = "a,\\,,b\\\\\n\\,,\\\nc,d\\\r\ne";
+        final CSVFormat format = formatWithEscaping.withIgnoreEmptyLines(false);
+        assertTrue(format.isEscapeCharacterSet());
         try (final Lexer parser = createLexer(code, format)) {
-            assertThat(parser.nextToken(new Token()), matches(TOKEN, "first"));
-            assertThat(parser.nextToken(new Token()), matches(TOKEN, "line"));
-            assertThat(parser.nextToken(new Token()), matches(EORECORD, ""));
-            assertThat(parser.nextToken(new Token()), matches(TOKEN, "second"));
-            assertThat(parser.nextToken(new Token()), matches(EORECORD, "line"));
-            assertThat(parser.nextToken(new Token()), matches(EORECORD, "third line "));
-            assertThat(parser.nextToken(new Token()), matches(TOKEN, "last"));
-            assertThat(parser.nextToken(new Token()), matches(EORECORD, " line "));
-            assertThat(parser.nextToken(new Token()), matches(EOF, ""));
+            assertThat(parser.nextToken(new Token()), matches(TOKEN, "a"));
+            assertThat(parser.nextToken(new Token()), matches(TOKEN, ","));
+            assertThat(parser.nextToken(new Token()), matches(EORECORD, "b\\"));
+            assertThat(parser.nextToken(new Token()), matches(TOKEN, ","));
+            assertThat(parser.nextToken(new Token()), matches(TOKEN, "\nc"));
+            assertThat(parser.nextToken(new Token()), matches(EORECORD, "d\r"));
+            assertThat(parser.nextToken(new Token()), matches(EOF, "e"));
+        }
+    }
+
+    // simple token with escaping not enabled
+    @Test
+    public void testBackslashWithoutEscaping() throws IOException {
+        /*
+         * file: a,\,,b \,,
+         */
+        final String code = "a,\\,,b\\\n\\,,";
+        final CSVFormat format = CSVFormat.DEFAULT;
+        assertFalse(format.isEscapeCharacterSet());
+        try (final Lexer parser = createLexer(code, format)) {
+            assertThat(parser.nextToken(new Token()), matches(TOKEN, "a"));
+            // an unquoted single backslash is not an escape char
+            assertThat(parser.nextToken(new Token()), matches(TOKEN, "\\"));
+            assertThat(parser.nextToken(new Token()), matches(TOKEN, ""));
+            assertThat(parser.nextToken(new Token()), matches(EORECORD, "b\\"));
+            // an unquoted single backslash is not an escape char
+            assertThat(parser.nextToken(new Token()), matches(TOKEN, "\\"));
+            assertThat(parser.nextToken(new Token()), matches(TOKEN, ""));
             assertThat(parser.nextToken(new Token()), matches(EOF, ""));
         }
     }
 
+    @Test
+    public void testBackspace() throws Exception {
+        try (final Lexer lexer = createLexer("character" + BACKSPACE + "NotEscaped", formatWithEscaping)) {
+            assertThat(lexer.nextToken(new Token()), hasContent("character" + BACKSPACE + "NotEscaped"));
+        }
+    }
+
     @Test
     public void testComments() throws IOException {
         final String code = "first,line,\n" + "second,line,tokenWith#no-comment\n" + "# comment line \n" +
@@ -172,45 +175,141 @@ public class LexerTest {
         }
     }
 
-    // simple token with escaping not enabled
     @Test
-    public void testBackslashWithoutEscaping() throws IOException {
-        /*
-         * file: a,\,,b \,,
-         */
-        final String code = "a,\\,,b\\\n\\,,";
-        final CSVFormat format = CSVFormat.DEFAULT;
-        assertFalse(format.isEscapeCharacterSet());
+    public void testCR() throws Exception {
+        try (final Lexer lexer = createLexer("character" + CR + "NotEscaped", formatWithEscaping)) {
+            assertThat(lexer.nextToken(new Token()), hasContent("character"));
+            assertThat(lexer.nextToken(new Token()), hasContent("NotEscaped"));
+        }
+    }
+
+    // From CSV-1
+    @Test
+    public void testDelimiterIsWhitespace() throws IOException {
+        final String code = "one\ttwo\t\tfour \t five\t six";
+        try (final Lexer parser = createLexer(code, CSVFormat.TDF)) {
+            assertThat(parser.nextToken(new Token()), matches(TOKEN, "one"));
+            assertThat(parser.nextToken(new Token()), matches(TOKEN, "two"));
+            assertThat(parser.nextToken(new Token()), matches(TOKEN, ""));
+            assertThat(parser.nextToken(new Token()), matches(TOKEN, "four"));
+            assertThat(parser.nextToken(new Token()), matches(TOKEN, "five"));
+            assertThat(parser.nextToken(new Token()), matches(EOF, "six"));
+        }
+    }
+
+    @Test // TODO is this correct? Do we expect <esc>BACKSPACE to be unescaped?
+    public void testEscapedBackspace() throws Exception {
+        try (final Lexer lexer = createLexer("character\\" + BACKSPACE + "Escaped", formatWithEscaping)) {
+            assertThat(lexer.nextToken(new Token()), hasContent("character" + BACKSPACE + "Escaped"));
+        }
+    }
+
+    @Test
+    public void testEscapedCharacter() throws Exception {
+        try (final Lexer lexer = createLexer("character\\aEscaped", formatWithEscaping)) {
+            assertThat(lexer.nextToken(new Token()), hasContent("character\\aEscaped"));
+        }
+    }
+
+    @Test
+    public void testEscapedControlCharacter() throws Exception {
+        // we are explicitly using an escape different from \ here
+        try (final Lexer lexer = createLexer("character!rEscaped", CSVFormat.DEFAULT.withEscape('!'))) {
+            assertThat(lexer.nextToken(new Token()), hasContent("character" + CR + "Escaped"));
+        }
+    }
+
+    @Test
+    public void testEscapedControlCharacter2() throws Exception {
+        try (final Lexer lexer = createLexer("character\\rEscaped", CSVFormat.DEFAULT.withEscape('\\'))) {
+            assertThat(lexer.nextToken(new Token()), hasContent("character" + CR + "Escaped"));
+        }
+    }
+
+    @Test
+    public void testEscapedCR() throws Exception {
+        try (final Lexer lexer = createLexer("character\\" + CR + "Escaped", formatWithEscaping)) {
+            assertThat(lexer.nextToken(new Token()), hasContent("character" + CR + "Escaped"));
+        }
+    }
+
+    @Test // TODO is this correct? Do we expect <esc>FF to be unescaped?
+    public void testEscapedFF() throws Exception {
+        try (final Lexer lexer = createLexer("character\\" + FF + "Escaped", formatWithEscaping)) {
+            assertThat(lexer.nextToken(new Token()), hasContent("character" + FF + "Escaped"));
+        }
+    }
+
+    @Test
+    public void testEscapedLF() throws Exception {
+        try (final Lexer lexer = createLexer("character\\" + LF + "Escaped", formatWithEscaping)) {
+            assertThat(lexer.nextToken(new Token()), hasContent("character" + LF + "Escaped"));
+        }
+    }
+
+    @Test
+    public void testEscapedMySqlNullValue() throws Exception {
+        // MySQL uses \N to symbolize null values. We have to restore this
+        try (final Lexer lexer = createLexer("character\\NEscaped", formatWithEscaping)) {
+            assertThat(lexer.nextToken(new Token()), hasContent("character\\NEscaped"));
+        }
+    }
+
+    @Test // TODO is this correct? Do we expect <esc>TAB to be unescaped?
+    public void testEscapedTab() throws Exception {
+        try (final Lexer lexer = createLexer("character\\" + TAB + "Escaped", formatWithEscaping)) {
+            assertThat(lexer.nextToken(new Token()), hasContent("character" + TAB + "Escaped"));
+        }
+
+    }
+
+    @Test
+    public void testEscapingAtEOF() throws Exception {
+        final String code = "escaping at EOF is evil\\";
+        try (final Lexer lexer = createLexer(code, formatWithEscaping)) {
+            assertThrows(IOException.class, () -> lexer.nextToken(new Token()));
+        }
+    }
+
+    @Test
+    public void testFF() throws Exception {
+        try (final Lexer lexer = createLexer("character" + FF + "NotEscaped", formatWithEscaping)) {
+            assertThat(lexer.nextToken(new Token()), hasContent("character" + FF + "NotEscaped"));
+        }
+    }
+
+    @Test
+    public void testIgnoreEmptyLines() throws IOException {
+        final String code = "first,line,\n" + "\n" + "\n" + "second,line\n" + "\n" + "\n" + "third line \n" + "\n" +
+                "\n" + "last, line \n" + "\n" + "\n" + "\n";
+        final CSVFormat format = CSVFormat.DEFAULT.withIgnoreEmptyLines();
         try (final Lexer parser = createLexer(code, format)) {
-            assertThat(parser.nextToken(new Token()), matches(TOKEN, "a"));
-            // an unquoted single backslash is not an escape char
-            assertThat(parser.nextToken(new Token()), matches(TOKEN, "\\"));
-            assertThat(parser.nextToken(new Token()), matches(TOKEN, ""));
-            assertThat(parser.nextToken(new Token()), matches(EORECORD, "b\\"));
-            // an unquoted single backslash is not an escape char
-            assertThat(parser.nextToken(new Token()), matches(TOKEN, "\\"));
-            assertThat(parser.nextToken(new Token()), matches(TOKEN, ""));
+            assertThat(parser.nextToken(new Token()), matches(TOKEN, "first"));
+            assertThat(parser.nextToken(new Token()), matches(TOKEN, "line"));
+            assertThat(parser.nextToken(new Token()), matches(EORECORD, ""));
+            assertThat(parser.nextToken(new Token()), matches(TOKEN, "second"));
+            assertThat(parser.nextToken(new Token()), matches(EORECORD, "line"));
+            assertThat(parser.nextToken(new Token()), matches(EORECORD, "third line "));
+            assertThat(parser.nextToken(new Token()), matches(TOKEN, "last"));
+            assertThat(parser.nextToken(new Token()), matches(EORECORD, " line "));
+            assertThat(parser.nextToken(new Token()), matches(EOF, ""));
             assertThat(parser.nextToken(new Token()), matches(EOF, ""));
         }
     }
 
-    // simple token with escaping enabled
     @Test
-    public void testBackslashWithEscaping() throws IOException {
-        /*
-         * file: a,\,,b \,,
-         */
-        final String code = "a,\\,,b\\\\\n\\,,\\\nc,d\\\r\ne";
-        final CSVFormat format = formatWithEscaping.withIgnoreEmptyLines(false);
-        assertTrue(format.isEscapeCharacterSet());
-        try (final Lexer parser = createLexer(code, format)) {
-            assertThat(parser.nextToken(new Token()), matches(TOKEN, "a"));
-            assertThat(parser.nextToken(new Token()), matches(TOKEN, ","));
-            assertThat(parser.nextToken(new Token()), matches(EORECORD, "b\\"));
-            assertThat(parser.nextToken(new Token()), matches(TOKEN, ","));
-            assertThat(parser.nextToken(new Token()), matches(TOKEN, "\nc"));
-            assertThat(parser.nextToken(new Token()), matches(EORECORD, "d\r"));
-            assertThat(parser.nextToken(new Token()), matches(EOF, "e"));
+    public void testIsMetaCharCommentStart() throws IOException {
+        try (final Lexer lexer = createLexer("#", CSVFormat.DEFAULT.withCommentMarker('#'))) {
+            final int ch = lexer.readEscape();
+            assertEquals('#', ch);
+        }
+    }
+
+    @Test
+    public void testLF() throws Exception {
+        try (final Lexer lexer = createLexer("character" + LF + "NotEscaped", formatWithEscaping)) {
+            assertThat(lexer.nextToken(new Token()), hasContent("character"));
+            assertThat(lexer.nextToken(new Token()), hasContent("NotEscaped"));
         }
     }
 
@@ -265,58 +364,59 @@ public class LexerTest {
         }
     }
 
-    // From CSV-1
     @Test
-    public void testDelimiterIsWhitespace() throws IOException {
-        final String code = "one\ttwo\t\tfour \t five\t six";
-        try (final Lexer parser = createLexer(code, CSVFormat.TDF)) {
-            assertThat(parser.nextToken(new Token()), matches(TOKEN, "one"));
-            assertThat(parser.nextToken(new Token()), matches(TOKEN, "two"));
+    public void testReadEscapeBackspace() throws IOException {
+        try (final Lexer lexer = createLexer("b", CSVFormat.DEFAULT.withEscape('\b'))) {
+            final int ch = lexer.readEscape();
+            assertEquals(BACKSPACE, ch);
+        }
+    }
+
+    @Test
+    public void testReadEscapeFF() throws IOException {
+        try (final Lexer lexer = createLexer("f", CSVFormat.DEFAULT.withEscape('\f'))) {
+            final int ch = lexer.readEscape();
+            assertEquals(FF, ch);
+        }
+    }
+
+    @Test
+    public void testReadEscapeTab() throws IOException {
+        try (final Lexer lexer = createLexer("t", CSVFormat.DEFAULT.withEscape('\t'))) {
+            final int ch = lexer.readEscape();
+            assertThat(lexer.nextToken(new Token()), matches(EOF, ""));
+            assertEquals(TAB, ch);
+        }
+    }
+
+    @Test
+    public void testSurroundingSpacesAreDeleted() throws IOException {
+        final String code = "noSpaces,  leadingSpaces,trailingSpaces  ,  surroundingSpaces  ,  ,,";
+        try (final Lexer parser = createLexer(code, CSVFormat.DEFAULT.withIgnoreSurroundingSpaces())) {
+            assertThat(parser.nextToken(new Token()), matches(TOKEN, "noSpaces"));
+            assertThat(parser.nextToken(new Token()), matches(TOKEN, "leadingSpaces"));
+            assertThat(parser.nextToken(new Token()), matches(TOKEN, "trailingSpaces"));
+            assertThat(parser.nextToken(new Token()), matches(TOKEN, "surroundingSpaces"));
             assertThat(parser.nextToken(new Token()), matches(TOKEN, ""));
-            assertThat(parser.nextToken(new Token()), matches(TOKEN, "four"));
-            assertThat(parser.nextToken(new Token()), matches(TOKEN, "five"));
-            assertThat(parser.nextToken(new Token()), matches(EOF, "six"));
+            assertThat(parser.nextToken(new Token()), matches(TOKEN, ""));
+            assertThat(parser.nextToken(new Token()), matches(EOF, ""));
         }
     }
 
     @Test
-    public void testEscapedCR() throws Exception {
-        try (final Lexer lexer = createLexer("character\\" + CR + "Escaped", formatWithEscaping)) {
-            assertThat(lexer.nextToken(new Token()), hasContent("character" + CR + "Escaped"));
+    public void testSurroundingTabsAreDeleted() throws IOException {
+        final String code = "noTabs,\tleadingTab,trailingTab\t,\tsurroundingTabs\t,\t\t,,";
+        try (final Lexer parser = createLexer(code, CSVFormat.DEFAULT.withIgnoreSurroundingSpaces())) {
+            assertThat(parser.nextToken(new Token()), matches(TOKEN, "noTabs"));
+            assertThat(parser.nextToken(new Token()), matches(TOKEN, "leadingTab"));
+            assertThat(parser.nextToken(new Token()), matches(TOKEN, "trailingTab"));
+            assertThat(parser.nextToken(new Token()), matches(TOKEN, "surroundingTabs"));
+            assertThat(parser.nextToken(new Token()), matches(TOKEN, ""));
+            assertThat(parser.nextToken(new Token()), matches(TOKEN, ""));
+            assertThat(parser.nextToken(new Token()), matches(EOF, ""));
         }
     }
 
-    @Test
-    public void testCR() throws Exception {
-        try (final Lexer lexer = createLexer("character" + CR + "NotEscaped", formatWithEscaping)) {
-            assertThat(lexer.nextToken(new Token()), hasContent("character"));
-            assertThat(lexer.nextToken(new Token()), hasContent("NotEscaped"));
-        }
-    }
-
-    @Test
-    public void testEscapedLF() throws Exception {
-        try (final Lexer lexer = createLexer("character\\" + LF + "Escaped", formatWithEscaping)) {
-            assertThat(lexer.nextToken(new Token()), hasContent("character" + LF + "Escaped"));
-        }
-    }
-
-    @Test
-    public void testLF() throws Exception {
-        try (final Lexer lexer = createLexer("character" + LF + "NotEscaped", formatWithEscaping)) {
-            assertThat(lexer.nextToken(new Token()), hasContent("character"));
-            assertThat(lexer.nextToken(new Token()), hasContent("NotEscaped"));
-        }
-    }
-
-    @Test // TODO is this correct? Do we expect <esc>TAB to be unescaped?
-    public void testEscapedTab() throws Exception {
-        try (final Lexer lexer = createLexer("character\\" + TAB + "Escaped", formatWithEscaping)) {
-            assertThat(lexer.nextToken(new Token()), hasContent("character" + TAB + "Escaped"));
-        }
-
-    }
-
     @Test
     public void testTab() throws Exception {
         try (final Lexer lexer = createLexer("character" + TAB + "NotEscaped", formatWithEscaping)) {
@@ -324,69 +424,11 @@ public class LexerTest {
         }
     }
 
-    @Test // TODO is this correct? Do we expect <esc>BACKSPACE to be unescaped?
-    public void testEscapedBackspace() throws Exception {
-        try (final Lexer lexer = createLexer("character\\" + BACKSPACE + "Escaped", formatWithEscaping)) {
-            assertThat(lexer.nextToken(new Token()), hasContent("character" + BACKSPACE + "Escaped"));
-        }
-    }
-
     @Test
-    public void testBackspace() throws Exception {
-        try (final Lexer lexer = createLexer("character" + BACKSPACE + "NotEscaped", formatWithEscaping)) {
-            assertThat(lexer.nextToken(new Token()), hasContent("character" + BACKSPACE + "NotEscaped"));
-        }
-    }
-
-    @Test // TODO is this correct? Do we expect <esc>FF to be unescaped?
-    public void testEscapedFF() throws Exception {
-        try (final Lexer lexer = createLexer("character\\" + FF + "Escaped", formatWithEscaping)) {
-            assertThat(lexer.nextToken(new Token()), hasContent("character" + FF + "Escaped"));
-        }
-    }
-
-    @Test
-    public void testFF() throws Exception {
-        try (final Lexer lexer = createLexer("character" + FF + "NotEscaped", formatWithEscaping)) {
-            assertThat(lexer.nextToken(new Token()), hasContent("character" + FF + "NotEscaped"));
-        }
-    }
-
-    @Test
-    public void testEscapedMySqlNullValue() throws Exception {
-        // MySQL uses \N to symbolize null values. We have to restore this
-        try (final Lexer lexer = createLexer("character\\NEscaped", formatWithEscaping)) {
-            assertThat(lexer.nextToken(new Token()), hasContent("character\\NEscaped"));
-        }
-    }
-
-    @Test
-    public void testEscapedCharacter() throws Exception {
-        try (final Lexer lexer = createLexer("character\\aEscaped", formatWithEscaping)) {
-            assertThat(lexer.nextToken(new Token()), hasContent("character\\aEscaped"));
-        }
-    }
-
-    @Test
-    public void testEscapedControlCharacter() throws Exception {
-        // we are explicitly using an escape different from \ here
-        try (final Lexer lexer = createLexer("character!rEscaped", CSVFormat.DEFAULT.withEscape('!'))) {
-            assertThat(lexer.nextToken(new Token()), hasContent("character" + CR + "Escaped"));
-        }
-    }
-
-    @Test
-    public void testEscapedControlCharacter2() throws Exception {
-        try (final Lexer lexer = createLexer("character\\rEscaped", CSVFormat.DEFAULT.withEscape('\\'))) {
-            assertThat(lexer.nextToken(new Token()), hasContent("character" + CR + "Escaped"));
-        }
-    }
-
-    @Test
-    public void testEscapingAtEOF() throws Exception {
-        final String code = "escaping at EOF is evil\\";
-        try (final Lexer lexer = createLexer(code, formatWithEscaping)) {
-            assertThrows(IOException.class, () -> lexer.nextToken(new Token()));
-        }
+    public void testTrimTrailingSpacesZeroLength() throws Exception {
+        final StringBuilder buffer = new StringBuilder("");
+        final Lexer lexer = createLexer(buffer.toString(), CSVFormat.DEFAULT);
+        lexer.trimTrailingSpaces(buffer);
+        assertThat(lexer.nextToken(new Token()), matches(EOF, ""));
     }
 }
